import argparse

from YamlSchemaAgent import generate
from YamlSchemaAgent.tags_pkg import get_yaml
from jsonschema.validators import validate


def main():
    parser = argparse.ArgumentParser(
        prog='Yaml Schema Generator',
        description='Generates a YAML schema with Validation capabilities',
        epilog='')
    parser.add_argument('-s', '--schema_file', required=False, default=None)
    parser.add_argument('-v', '--validate', required=False, default=None)
    parser.add_argument('-g', '--generate_from', required=False, default=None)
    parser.add_argument('-o', '--output', required=False, default=None)
    args = parser.parse_args()
    schema = None
    yaml_context = get_yaml()
    if args.schema_file:
        schema = yaml_context.safe_load(args.schema_file)
    if args.generate_from:
        schema = generate.run(yaml_context=yaml_context, schema_loc=args.generate_from, output=args.output)
    if args.validate:
        input = yaml_context.safe_load(open(args.validate).read())
        assert validate(instance=input, schema=schema) is None
        print("All Done")


if __name__ == '__main__':
    main()



def add_str():
    return dict(type="string")


def add_int():
    return dict(type="integer")


def add_list():
    return dict(type="array")


def add_nonetype():
    return dict(type='null')


def add_number():
    return dict(type='number')


def add_float():
    return add_number()


def add_bool():
    return dict(type='boolean')



import json
import yaml
from yaml import Loader, ScalarNode


class YSGTag(yaml.YAMLObject):
    def __init__(self, yaml_tag, tag_type):
        self.yaml_tag = yaml_tag
        self.schema = dict(type=tag_type)

    def __repr__(self):
        return json.dumps(self.schema)

    def from_yaml(self, loader: Loader, node):
        if isinstance(node, ScalarNode) and node.value == '':
            return self.schema
        self.schema.update(loader.construct_mapping(node, deep=True))
        return self.schema

    def to_yaml(self, dumper, data):
        return dumper.represent_scalar(self.yaml_tag, data)


def get_yaml():
    string_tag = YSGTag(yaml_tag=u'!STRING', tag_type='string')
    number_tag = YSGTag(yaml_tag=u'!NUMBER', tag_type='number')
    integer_tag = YSGTag(yaml_tag=u'!INTEGER', tag_type='integer')
    array_tag = YSGTag(yaml_tag=u'!ARRAY', tag_type='array')
    yaml.SafeLoader.add_constructor(string_tag.yaml_tag, string_tag.from_yaml)
    yaml.SafeDumper.add_multi_representer(string_tag, string_tag.to_yaml)
    yaml.SafeLoader.add_constructor(number_tag.yaml_tag, number_tag.from_yaml)
    yaml.SafeDumper.add_multi_representer(number_tag, number_tag.to_yaml)
    yaml.SafeLoader.add_constructor(integer_tag.yaml_tag, integer_tag.from_yaml)
    yaml.SafeDumper.add_multi_representer(integer_tag, integer_tag.to_yaml)
    yaml.SafeLoader.add_constructor(array_tag.yaml_tag, array_tag.from_yaml)
    yaml.SafeDumper.add_multi_representer(array_tag, array_tag.to_yaml)
    return yaml



import json

from YamlSchemaAgent import schema_types


def decode_base_types(v):
    print(f'add_{type(v).__name__.lower()}')
    if hasattr(schema_types, f'add_{type(v).__name__.lower()}'):
        return getattr(schema_types, f'add_{type(v).__name__.lower()}')()


def walk_and_decode(data, jsonschema):
    for k, v in data.items():
        if isinstance(v, dict):
            if 'type' not in v:
                jsonschema[k] = {
                    "type": "object",
                    "properties": {}
                }
                walk_and_decode(v, jsonschema[k]['properties'])
            else:
                jsonschema[k] = v
        else:
            jsonschema[k] = decode_base_types(v)


def run(yaml_context, schema_loc, output=None):
    schema = yaml_context.safe_load(open(schema_loc).read())
    json_schema = {
        "type": "object",
        "properties": {
        },
    }
    walk_and_decode(schema, json_schema['properties'])
    print("***"*20 + "GENERATED JSON SCHEMA" + "***"*20 )
    print(json.dumps(json_schema, indent=2))
    with open(output or 'ysg_'+ schema_loc.split('/')[-1], 'w') as gen_schema:
        gen_schema.write(json.dumps(json_schema, indent=4))
    return json_schema



from setuptools import setup, find_packages

setup(
    name='yaml-schema-agent',
    version='0.1.1',
    url='https://github.com/nayak-swastik/yaml-schema',
    description='Project to Generate Schema from/to YAML and Also Validate',
    author='Swastik.Nayak',
    author_email='swastik.nayak.eu@gmail.com',
    packages=find_packages(),
    install_requires=['jsonschema'],
    long_description="Currently under development",
    entry_points={
        'console_scripts': ['ysg=YamlSchemaAgent.entry_point:main'],
    }
)
